#' ---
#' title: "BDOTS Analysis"
#' author: ""
#' date: ""
#' ---

#### Install and run packages ##############################################################

#install.packages("librarian")
librarian::shelf(tidyr, ggplot2, readxl, tidyverse, grid, gridExtra)

#Install BDOTS version 1.2.0 if necessary
#package = "https://cran.r-project.org/package=bdots&version=1.2.0"
#utils::install.packages(pkgs = package, repos = NULL)
#OR Install by dowloading package and installing manually from https://cran.r-project.org/package=bdots&version=1.2.0

library(bdots)
#view the Bdots vignette for more explanation (doesn't work with updated BDOTS package)
print(browseVignettes(package="bdots"))
citation("bdots")

#### Prepare Data ######################################################################

#Set Working Directory in R Studio
#R Studio File and Data File should be in the same folder 
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
getwd()

#Load File & Setup Data Frame
BDOTSWide<- read_excel("BDOTS All Data.xlsx")
BDOTS<-pivot_longer(BDOTSWide, cols=num_range("",1:50), names_to="Time", values_to="Accuracy")
BDOTS$Time<-as.numeric(BDOTS$Time)
table(BDOTS$Group, BDOTS$Experiment) # Check Experiments and Groups
#view(BDOTS) #Check data if necessary

#If needed, apply filter for Experiment, Group, etc.
BDOTS<-filter(BDOTS, Experiment == "Henry17")
table(BDOTS$Group, BDOTS$Experiment)

#Get Rid of NAs
data_subset <- BDOTS[ , c("Accuracy")]
BDOTS <- BDOTS[complete.cases(data_subset), ]
table(BDOTS$Group, BDOTS$Experiment)
#view(BDOTS)

############################################################################################
#### Load in Polynomial 2 function #########################################################
#### May not be necessary if using updated versions of BDOTS ###############################
############################################################################################

polynomial2 <- function(dat, y, time, degree, raw = TRUE, params = NULL, ...)  {
  polyPars <- function(dat, y, time, degree, raw) {
    if (var(dat[[y]]) == 0) {
      return(NULL)
    }
    pp <- lm(dat[[y]] ~ poly(dat[[time]], degree = degree, raw = raw))
    setNames(coef(pp), c(paste0("beta", seq(degree + 1L))))
  }
  if (is.null(params)) {
    params <- polyPars(dat, y, time, degree, raw)
  } else {
    if (length(params) != degree + 1L) stop("poly requires that number of params matches degree + 1 (for intercept)")
    if (!all(names(params) %in% paste0("beta", seq(degree + 1L)))) {
      stop("polynomial parameters must be labeled beta1, ..., beta[degree + 1]")
    }
  }
  ## Return NA list if var(y) is 0
  if (is.null(params)) {
    return(NULL)
  }
  time_names <- paste0("(Time^", seq(degree + 1L) - 1L , ")")
  ff <- paste(names(params), time_names, sep = "*", collapse = "+")
  ff <- str2lang(ff)
  y <- str2lang(y)
  ff <- bquote(.(y) ~ .(ff))
  attr(ff, "parnames") <- names(params)
  return(list(formula = ff, params = params))
}

############################################################################################
#### Begin BDOTS Analysis ##################################################################
############################################################################################

#### Fitting Step #########################################################################

#Complete Initial Fit
BDfit <- bdotsFit(data = BDOTS,
                  subject = "Subject",
                  time = "Time",
                  y = "Accuracy",
                  group = "Group",
                  curveType = polynomial2(degree=3),
                  cores = 2)

#Check Fit for participants
#Fit will often be 2 (see vignette) given sample size of 1 at each time point for each participant
table(BDfit$fitCode)  
head(coef(BDfit)) #Return a matrix of coefficients from the model fits (see vignette)
plot(BDfit[1:4,]) #Visualize (first 4) individual participant curves


#### Refitting Step ########################################################################

#Run Refit to see if fit can be improved
BDrefit <- bdotsRefit(BDfit, fitCode = 1L, quickRefit = TRUE)
table(BDrefit$fitCode)

## Manual refit if desired
#refit <- bdotsRefit(BDfit, fitCode = 1L)

## Remove all failed curve fits from refit
BDrefit <- bdRemove(BDrefit, fitCode = 6L)
table(BDrefit$fitCode)


############################################################################################
#### Bootstrapping Groups 1, 2 #############################################################
############################################################################################

table(BDOTS$Group) #Remind of the group names
BDBoot1 <- bdotsBoot(formula = Accuracy ~ Group(PInoEI, PIwEI),
                   bdObj = BDrefit,
                   Niter = 1000,
                   alpha = 0.05,
                   padj = "oleson",
                   cores = 2)
summary(BDBoot1)
plot(BDBoot1, ciBands=TRUE, plotDiffs=TRUE) #Default Plot from BDOTS


#### Plot in GGPlot if preferred ##############################################################

#Create CSV for Plotting Data
dir.create("BDOTS Output/") #Create Directory to hold output files
writeCSV(BDBoot1, "BDOTS Output/Bootstraps1.csv", alpha=0.05)
PlotDat1<-read_csv("BDOTS Output/Bootstraps1.csv")

#Read CSV Data Into PlotDat1 Object
PlotDat1<-pivot_longer(PlotDat1, -c(Time,Significant), names_sep = "\\ - ", names_to = c("Group", "stat"))
PlotDat1<-pivot_wider(PlotDat1, id_cols = c("Time", "Significant", "Group"), names_from= "stat", values_from="value")
PlotDat1$LowerCI<-PlotDat1$'Lower CI'
PlotDat1$UpperCI<-PlotDat1$'Upper CI'

BDBoot1$sigTime

#Create Data Frame for areas of significance
start1<-BDBoot1$sigTime[,1]
end1<-BDBoot1$sigTime[,2]
if(length(start1)!=0){SE<-data.frame(Group="", start1=start1, end1=end1)}

#Define Scale/ Limits / Labels
scale<-scale_x_continuous(breaks=seq(0,50, 10))
limits<-expand_limits(x=c(1,50), y=c(0,1))
labels<-labs(x="Item", y="Accuracy")

GG1<-filter(PlotDat1, Group !="diff")%>% #"diff" is the group differences (right frame of the bdots plot)
  arrange(Group) %>%
  mutate(Group = factor(Group, levels=c("PInoEI", "PIwEI"))) %>%
  ggplot(aes(Time, Fit, group=Group, fill=Group, color=Group))+
  geom_line(aes(y=Fit))+
  geom_ribbon(aes(ymin=LowerCI, ymax=UpperCI), color=NA , alpha = .1) +
#  stat_summary(data=filter(BDOTS, Group=="PInoEI"|Group=="PIwEI"), 
#               aes(x=Time, y=Accuracy, color=Group), 
#               fun=mean, geom="line", linetype="dashed")+ #Stat Summary for Raw Data
  theme_bw(base_size=10)+labels+
  scale_color_manual(values=c("#F8766D", "#00BA38"), 
                     labels = c("PInoEI" = "-P-EI", "PIwEI" = "-P+EI")) +
  scale_fill_manual(values=c("#F8766D", "#00BA38"), 
                    labels = c("PInoEI" = "-P-EI", "PIwEI" = "-P+EI")) +
  if(length(start1)!=0){geom_rect(data=SE, aes(NULL, NULL, xmin = start1, xmax = end1, ymin = 0, ymax = 1), color=NA, fill="gray", alpha=.5)}
GG1

#################################################################################################
####### Bootstrapping 1, 3 ######################################################################
#################################################################################################

####Repeat the Bootstrapping process for Groups 1, 3  ###########################################

table(BDOTS$Group) #Remind of the group names
BDBoot2 <- bdotsBoot(formula = Accuracy ~ Group(PInoEI, PIwPnoEI),
                     bdObj = BDrefit,
                     Niter = 1000,
                     alpha = 0.05,
                     padj = "oleson",
                     cores = 2)
summary(BDBoot2)
plot(BDBoot2, ciBands=TRUE, plotDiffs=FALSE)


####Repeat the Plotting process for Groups 1, 3  ###########################################

#Create CSV for Plotting Data
writeCSV(BDBoot2, "BDOTS Output/Bootstraps2.csv", alpha=0.05)
PlotDat2<-read_csv("BDOTS Output/Bootstraps2.csv")

#Read CSV into PlotDat2 Object
PlotDat2<-pivot_longer(PlotDat2, -c(Time,Significant), names_sep = "\\ - ", names_to = c("Group", "stat"))
PlotDat2<-pivot_wider(PlotDat2, id_cols = c("Time", "Significant", "Group"), names_from= "stat", values_from="value")
PlotDat2$LowerCI<-PlotDat2$'Lower CI'
PlotDat2$UpperCI<-PlotDat2$'Upper CI'

#Create Data Frame for areas of significance
start2<-BDBoot2$sigTime[,1]
end2<-BDBoot2$sigTime[,2]
if(length(start2)!=0){SE<-data.frame(Group="", start2=start2, end2=end2)}

#Create Plot
GG2<-filter(PlotDat2, Group !="diff")%>% 
  arrange(Group) %>%
  mutate(Group = factor(Group, levels=c("PInoEI", "PIwPnoEI"))) %>%
  ggplot(aes(Time, Fit, group=Group, fill=Group, color=Group))+
  geom_line(aes(y=Fit))+
  geom_ribbon(aes(ymin=LowerCI, ymax=UpperCI), color=NA , alpha = .1) +
  theme_bw(base_size=10)+labels+
  #scale_fill_brewer(aesthetics=c("color", "fill"), palette="Set1")+
#  stat_summary(data=filter(BDOTS, Group=="PInoEI"|Group=="PIwPnoEI"), 
#               aes(x=Time, y=Accuracy, color=Group), 
#               fun=mean, geom="line", linetype="dashed")+ #raw data
  scale_color_manual(values=c("#F8766D", "#00BFC4"), 
                     labels = c("PInoEI" = "-P-EI", "PIwPnoEI" = "+P-EI")) +
  scale_fill_manual(values=c("#F8766D", "#00BFC4"), 
                    labels = c("PInoEI" = "-P-EI", "PIwPnoEI" = "+P-EI")) +
  if(length(start2)!=0){geom_rect(data=SE, aes(NULL, NULL, xmin = start2, xmax = end2, ymin = 0, ymax = 1), color=NA, fill="gray", alpha=.5)}
GG2


#################################################################################################
####### Bootstrapping 2, 3 ######################################################################
#################################################################################################

####### Repeat bootstrapping process for Groups 2, 3 ############################################

table(BDOTS$Group) #Remind of the group names
BDBoot3 <- bdotsBoot(formula = Accuracy ~ Group(PIwEI, PInoEI),
                     bdObj = BDrefit,
                     Niter = 1000,
                     alpha = 0.05,
                     padj = "oleson",
                     cores = 2)
summary(BDBoot3)
plot(BDBoot3, ciBands=TRUE, plotDiffs=FALSE)

####Repeat the Plotting process for Groups 2, 3 ###########################################

#Create CSV for Plotting Data
writeCSV(BDBoot3, "BDOTS Output/Bootstraps3.csv", alpha=0.05)
PlotDat3<-read_csv("BDOTS Output/Bootstraps3.csv")

#Read CSV into PlotDat3 Object  ###########################################
PlotDat3<-pivot_longer(PlotDat3, -c(Time,Significant), names_sep = "\\ - ", names_to = c("Group", "stat"))
PlotDat3<-pivot_wider(PlotDat3, id_cols = c("Time", "Significant", "Group"), names_from= "stat", values_from="value")
PlotDat3$LowerCI<-PlotDat3$'Lower CI'
PlotDat3$UpperCI<-PlotDat3$'Upper CI'

#Create Data Frame for Rectangles
start3<-BDBoot3$sigTime[,1]
end3<-BDBoot3$sigTime[,2]
if(length(start3)!=0){SE<-data.frame(Group="", start3=start3, end3=end3)}

GG3<-filter(PlotDat3, Group !="diff")%>% 
  arrange(Group) %>%
  mutate(Group = factor(Group, levels=c("PIwEI", "PInoEI"))) %>%
  ggplot(aes(Time, Fit, group=Group, fill=Group, color=Group))+
  geom_line(aes(y=Fit))+
  geom_ribbon(aes(ymin=LowerCI, ymax=UpperCI), color=NA , alpha = .1) +
  theme_bw(base_size=10)+labels+
#  stat_summary(data=filter(BDOTS, Group=="PIwEI"|Group=="PInoEI"), 
#               aes(x=Time, y=Accuracy, color=Group), 
#               fun=mean, geom="line", linetype="dashed")+ #Raw Data
  scale_color_manual(values=c("#00BA38", "#00BFC4"), 
                     labels = c("PIwEI" = "PI+EI", "PInoEI" = "PI-EI"))  +
  scale_fill_manual(values=c("#00BA38", "#00BFC4"), 
                    labels = c("PIwEI" = "PI+EI", "PInoEI" = "PI-EI")) +
  if(length(start3)!=0){geom_rect(data=SE, aes(NULL, NULL, xmin = start3, xmax = end3, ymin = 0, ymax = 1), color=NA, fill="gray", alpha=.5)}
GG3

########################################################################################
####### FULL PLOT ######################################################################
########################################################################################

#Arrange desired plots into one figure
#In this example, I only want 2 figures in the plot since I'm only really concerned with 2 of the comparisons
FullPlot<-grid.arrange(GG2+limits+theme_bw(base_size=15), GG1+limits+theme_bw(base_size=15),
                       #blank, blank, 
                       ncol=2, nrow=1)
#Save full plot
ggsave(plot=FullPlot, filename="BDOTS Output/FullPlot.pdf", device="pdf", 
       width=15, height =9, units="in", dpi=1200)


#### Or, Alternatively ################################################################

#Arrange desired plots into one figure
#In this case, I want all three plots with the bottom one centered below the top 2
FullPlot2<-grid.arrange(GG2+limits+theme_bw(base_size=15), GG1+limits+theme_bw(base_size=15),
                        GG3+limits+theme_bw(base_size=15),
                        layout_matrix=rbind(c(1,1,2,2),c(NA,3,3,NA)))
ggsave(plot=FullPlot2, filename="BDOTS Output/FullPlot2.pdf", device="pdf", 
       width=15, height =9, units="in", dpi=1200)
